@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2014 Massachusetts Institute of Technology.
+/* Copyright (C) 2005-2015 Massachusetts Institute of Technology.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,13 +24,13 @@ namespace meep {
 
 typedef void (*bicgstab_op)(const realnum *x, realnum *y, void *data);
 
-int bicgstabL(const int L, 
-	      const int n, realnum *x,
-	      bicgstab_op A, void *Adata, const realnum *b,
-	      const double tol, 
-	      int *iters, // input *iters = max iters, output = actual iters
-	      realnum *work, // if you pass work=NULL, bicgstab returns nwork
-	      const bool quiet);
+ptrdiff_t bicgstabL(const int L,
+              const size_t n, realnum *x,
+              bicgstab_op A, void *Adata, const realnum *b,
+              const double tol,
+              int *iters, // input *iters = max iters, output = actual iters
+              realnum *work, // if you pass work=NULL, bicgstab returns nwork
+              const bool quiet);
 
 } // namespace meep
 
